@@ -257,6 +257,15 @@ check_residualness <- function(curve_set) {
 #' @param ... For expert use only.
 #' @return An object of class \code{curve_set}.
 #' @export
+#' @examples
+#' # 1d
+#' cset <- create_curve_set(list(r=1:10, obs=matrix(runif(10*5), ncol=5)))
+#' plot(cset)
+#' # 2d
+#' cset <- create_curve_set(list(r=data.frame(x=c(rep(1:3, 3), 4), y=c(rep(1:3, each=3), 1),
+#'                                            width=1, height=1),
+#'                               obs=matrix(runif(10*5), ncol=5)))
+#' plot(cset)
 create_curve_set <- function(curve_set, ...) {
   check_curve_set_content(curve_set, ...)
   is1obs <- is.vector(curve_set[['obs']])
@@ -270,6 +279,7 @@ create_curve_set <- function(curve_set, ...) {
   cset <- list(r = r, funcs = funcs, is1obs = is1obs)
   if(!is.null(curve_set[['theo']])) cset$theo <- curve_set[['theo']]
   class(cset) <- 'curve_set'
+  if(!is.vector(r)) class(cset) <- c('curve_set2d', class(cset))
   cset
 }
 
@@ -306,7 +316,6 @@ print.curve_set <- function(x, ...) {
 #' @export
 #' @importFrom graphics plot
 #' @importFrom graphics lines
-#' @importFrom grDevices grey
 #' @importFrom graphics axis
 #' @importFrom graphics abline
 #' @importFrom ggplot2 ggplot
@@ -314,9 +323,10 @@ print.curve_set <- function(x, ...) {
 #' @importFrom ggplot2 scale_x_continuous
 #' @importFrom ggplot2 scale_y_continuous
 #' @importFrom ggplot2 labs
+#' @importFrom ggplot2 geom_vline
 plot.curve_set <- function(x, plot_style = c("ggplot2", "basic"),
                            ylim, xlab = "r", ylab = "obs", main = NULL,
-                           col_obs = 1, col_sim = grDevices::grey(0.7),
+                           col_obs = 1, col_sim = 'grey70',
                            base_size = 11, ...) {
   plot_style <- match.arg(plot_style)
   funcs <- curve_set_funcs(x)
@@ -335,7 +345,9 @@ plot.curve_set <- function(x, plot_style = c("ggplot2", "basic"),
   switch(plot_style,
          ggplot2 = {
              df <- data.frame(r = rvalues, f = c(funcs), id = rep(1:ncol(funcs), each=nrow(funcs)))
-             p <- ( ggplot(data=df) + geom_line(aes_(x = ~r, y = ~f, group = ~id))
+             p <- ( ggplot()
+                   + geom_line(data=df[df$id==1,], aes_(x = ~r, y = ~f, group = ~id), col=col_obs)
+                   + geom_line(data=df[df$id!=1,], aes_(x = ~r, y = ~f, group = ~id), col=col_sim)
                    + scale_y_continuous(name = ylab, limits = ylim)
                    + labs(title=main)
                    + ThemePlain(base_size=base_size))
@@ -365,6 +377,32 @@ plot.curve_set <- function(x, plot_style = c("ggplot2", "basic"),
          })
 }
 
+#' Plot method for the class 'curve_set2d'
+#'
+#' Plot method for the class 'curve_set2d', i.e. two-dimensional functions
+#'
+#' @param x An \code{curve_set2d} object
+#' @inheritParams plot.curve_set
+#' @param idx Indices of functions to plot for 2d plots.
+#' @param ... Additional parameters to be passed to plot and lines.
+#' @inheritParams plot.global_envelope
+#'
+#' @export
+#' @importFrom ggplot2 ggplot
+#' @importFrom ggplot2 facet_wrap
+#' @importFrom ggplot2 labs
+#' @examples
+#' data(abide_9002_23)
+#' plot(abide_9002_23$curve_set, idx=c(1, 27))
+plot.curve_set2d <- function(x, idx=1, base_size = 11, ...) {
+  funcs <- curve_set_funcs(x)
+  rdf <- curve_set_rdf(x)
+  data <- do.call(rbind, lapply(idx, function(i) data.frame(idx=i, f=funcs[,i])))
+  df <- cbind(rdf, data)
+  return(ggplot() + choose_geom(df, varfill='f') +
+           facet_wrap("idx") + labs(x="x", y="y", fill=""))
+}
+
 # Combine curve sets.
 #
 # Combine curve sets to a one curve set, e.g. for testing by means of several test functions.
@@ -471,6 +509,14 @@ curve_set_rdf <- function(curve_set) {
   else r
 }
 
+curve_set_is1d <- function(curve_set) {
+  !inherits(curve_set, 'curve_set2d')
+}
+
+curve_set_is2d <- function(curve_set) {
+  inherits(curve_set, 'curve_set2d')
+}
+
 curve_set_isresidual <- function(curve_set) {
   x <- curve_set[['is_residual']]
   if(is.null(x)) FALSE
