@@ -14,9 +14,9 @@
 #' 2) The deviation_test function then
 #'\itemize{
 #'   \item Crops the functions to the chosen range of distances [r_min, r_max].
-#'   \item If the curve_set does not consist of residuals, i.e. curve_set$is_residual
-#'     is FALSE (or does not exists), then the residuals d_i(r) = T_i(r) - T_0(r) are
-#'     calculated, where T_0(r) is the expectation of T(r) under the null hypothesis.
+#'   \item If the curve_set does not consist of residuals (see \code{\link{residual}}),
+#'     then the residuals d_i(r) = T_i(r) - T_0(r) are calculated, where T_0(r) is the
+#'     expectation of T(r) under the null hypothesis.
 #'     If use_theo = TRUE, the theoretical value given in the curve_set$theo is used for
 #'     as T_0(r), if it is given. Otherwise, T_0(r) is estimated by the mean of T_j(r),
 #'     j=2,...,nsim+1.
@@ -93,7 +93,7 @@ deviation_test <- function(curve_set, r_min = NULL, r_max = NULL,
   curve_set <- residual(curve_set, use_theo = use_theo)
   curve_set <- scale_curves(curve_set, scaling = scaling)
   devs <- deviation(curve_set, measure = measure)
-  p <- estimate_p_value(devs)
+  p <- estimate_p_value(devs[1], devs[-1])
   if(savedevs) res <- list(p=p, devs=devs, call=match.call())
   else res <- list(p=p, call=match.call())
   class(res) <- 'deviation_test'
